@@ -15,7 +15,7 @@
 
 Well, at least begin testing some of the functionality
 
-$Id: testZopeGuards.py 39645 2005-10-26 17:12:35Z jim $
+$Id: testZopeGuards.py 40438 2005-12-01 01:55:42Z tseaver $
 """
 
 import os, sys
@@ -419,10 +419,13 @@ class TestActualPython(GuardTestCase):
     _old_mgr = _old_policy = _marker = []
 
     def setUp(self):
-        pass
+        self._wrapped_dicts = []
 
     def tearDown( self ):
         self._restorePolicyAndManager()
+        for munged, orig in self._wrapped_dicts:
+            munged.update(orig)
+        del self._wrapped_dicts
 
     def _initPolicyAndManager(self, manager=None):
         from AccessControl.SecurityManagement import get_ident
@@ -668,6 +671,8 @@ print foo(**kw)
     # tell whether they're executed.
     def _wrap_replaced_dict_callables(self, d):
         import __builtin__
+        orig = d.copy()
+        self._wrapped_dicts.append((d, orig))
         for k, v in d.items():
             if callable(v) and v is not getattr(__builtin__, k, None):
                 d[k] = FuncWrapper(k, v)
