@@ -12,7 +12,7 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 37951 2005-08-16 10:40:48Z andreasjung $
+$Id: ZCatalog.py 71167 2006-11-17 19:51:12Z lra $
 """
 
 from warnings import warn
@@ -35,6 +35,7 @@ from ZODB.POSException import ConflictError
 import transaction
 from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
+from Products.PluginIndexes.interfaces import IPluggableIndex
 from Products.PluginIndexes.TextIndex import Splitter
 from zLOG import LOG
 from zope.interface import implements
@@ -231,7 +232,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             for url in urls:
                 obj = self.resolve_path(url)
-                if not obj:
+                if obj is None:
                     obj = self.resolve_url(url, REQUEST)
                 if obj is not None:
                     self.catalog_object(obj, url)
@@ -296,7 +297,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             p = paths[i]
             obj = self.resolve_path(p)
             
-            if not obj and hasattr(self, 'REQUEST'):
+            if obj is None and hasattr(self, 'REQUEST'):
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
@@ -587,7 +588,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             # we should commit a subtransaction if our threshhold is
             # exceeded within the boundaries of the current transaction.
             if self._v_total > self.threshold:
-                transaction.savepoint()
+                transaction.savepoint(optimistic=True)
                 self._p_jar.cacheGC()
                 self._v_total = 0
                 if pghandler:
@@ -614,12 +615,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     def getobject(self, rid, REQUEST=None):
         """Return a cataloged object given a 'data_record_id_'
         """
-        obj = self.aq_parent.unrestrictedTraverse(self.getpath(rid))
-        if not obj:
-            if REQUEST is None:
-                REQUEST=self.REQUEST
-            obj = self.resolve_url(self.getpath(rid), REQUEST)
-        return obj
+        return self.aq_parent.unrestrictedTraverse(self.getpath(rid))
 
     def getMetadataForUID(self, uid):
         """return the correct metadata given the uid, usually the path"""
@@ -910,7 +906,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         self.pgthreshold = pgthreshold
         if RESPONSE:
             RESPONSE.redirect(
-                URL1 + '/manage_main?manage_tabs_message=Catalog%20Changed')
+                URL1 + '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')
 
     def _getProgressThreshold(self):
         if not hasattr(self, 'pgthreshold'):
@@ -933,7 +929,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             bases = [str(name) for name in idx.__class__.__bases__]
             found = False
 
-            if idx.meta_type  == 'PathIndex':
+            if idx.meta_type == 'PathIndex':
                 found = True
             else:
                 for base in bases:
@@ -958,6 +954,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
                 if indexed_attrs:
                     setattr(new_idx, 'indexed_attrs', indexed_attrs)
+
                 if idx.meta_type == 'DateRangeIndex':
                     setattr(new_idx, '_since_field',  since_field)
                     setattr(new_idx, '_until_field', until_field)
@@ -984,7 +981,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         # this interface by that name.  Bleah
 
         products = ObjectManager.all_meta_types(self, interfaces=(
-            PluggableIndexInterface,))
+            PluggableIndexInterface, IPluggableIndex))
 
         p = None
 
